import { Module } from '@nestjs/common';
import { PipelineTasksService } from './pipeline-tasks.service';
import { PipelineTasksResolver } from './pipeline-tasks.resolver';
import { TypeOrmModule } from '@nestjs/typeorm';
import { PipelineTask } from './pipeline-task.entity';
import { Pipeline } from '../pipelines/pipeline.entity';
import { ReposModule } from '../repos/repos.module';
import { RedisModule } from 'nestjs-redis';
import { BullModule } from '@nestjs/bull';
import { PipelineTaskConsumer } from './pipeline-task.consumer';
import { PIPELINE_TASK_QUEUE } from './pipeline-tasks.constants';
import { PipelineTaskLogsService } from './pipeline-task-logs.service';
import { PubSubModule } from '../commons/pub-sub/pub-sub.module';
import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { PipelineTaskRunner } from './pipeline-task.runner';
import { spawn } from 'child_process';

@Module({
  imports: [
    TypeOrmModule.forFeature([PipelineTask, Pipeline]),
    BullModule.registerQueue({
      name: PIPELINE_TASK_QUEUE,
    }),
    PubSubModule.forFeature(),
    RedisModule,
    ReposModule,

    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      useFactory: (configService: ConfigService) => ({
        uri: configService.get<string>('db.rabbitmq.uri'),
        exchanges: [
          {
            name: 'new-pipeline-task',
            type: 'fanout',
            options: {
              durable: true,
            },
          },
          {
            name: 'stop-pipeline-task',
            type: 'fanout',
            options: {
              durable: true,
            },
          },
          {
            name: 'update-pipeline-task',
            type: 'fanout',
            options: {
              durable: false,
            },
          },
        ],
      }),
      inject: [ConfigService],
    }),
  ],
  providers: [
    PipelineTasksService,
    PipelineTasksResolver,
    PipelineTaskConsumer,
    PipelineTaskLogsService,
    PipelineTaskRunner,
    {
      provide: 'spawn',
      useValue: spawn,
    },
  ],
  exports: [PipelineTasksService],
})
export class PipelineTasksModule {}
